import { createCert } from '@kevisual/router/sign';
import path from 'node:path';
import fs from 'node:fs';
import { AssistantConfig } from '../config/index.ts';
import { checkFileExists } from '../file/index.ts';
import { chalk } from '@/module/chalk.ts';
type Attributes = {
  name: string;
  value: string;
};
type AltNames = {
  type: number;
  value?: string;
  ip?: string;
};
export class HttpsPem {
  assistantConfig: AssistantConfig;
  key: string;
  cert: string;
  constructor(assistantConfig: AssistantConfig) {
    this.assistantConfig = assistantConfig;
    const { key, cert } = this.getCert();
    this.key = key;
    this.cert = cert;
  }
  getPemDir() {
    const configDir = this.assistantConfig.configPath?.configDir || process.cwd();
    const pemDir = path.join(configDir, 'pem');
    if (!checkFileExists(pemDir)) {
      fs.mkdirSync(pemDir, { recursive: true });
    }
    return pemDir;
  }
  getCert() {
    const pemDir = this.getPemDir();
    const pemPath = {
      key: path.join(pemDir, 'https-private-key.pem'),
      cert: path.join(pemDir, 'https-cert.pem'),
    };
    if (!checkFileExists(pemPath.key) || !checkFileExists(pemPath.cert)) {
      const { key, cert } = this.createCert();
      fs.writeFileSync(pemPath.key, key);
      fs.writeFileSync(pemPath.cert, cert);
      console.log(chalk.green('证书创建成功'))
      return {
        key,
        cert,
      };
    }
    const key = fs.readFileSync(pemPath.key, 'utf-8');
    const cert = fs.readFileSync(pemPath.cert, 'utf-8');
    return {
      key,
      cert,
    };
  }
  createCert(attrs?: Attributes[], altNames?: AltNames[]) {
    const attributes = attrs || [];
    const altNamesList = altNames || [];
    const { key, cert } = createCert(
      [
        {
          name: 'commonName',
          value: 'localhost',
        },
        {
          name: 'organizationName',
          value: 'kevisual',
        },
        ...attributes,
      ],
      altNamesList,
    );
    return {
      key,
      cert,
    };
  }
}
